"use strict";
appServices
  .factory('GroupsService', ["$resource", "ApiEndPoint", function ($resource, ApiEndPoint) {

    /**
     * @type {{name: string, privacy: string, members: Array, image: string, phrase: string}}
     */
    var groupForm = newGroupForm();

    function resetGroupForm() {
      groupForm = newGroupForm();
    }

    /**
     *
     * @returns {{name: string, privacy: string, members: Array, image: string, phrase: string}}
     */
    function newGroupForm() {
      return {
        name: "",
        privacy: "Public",
        members: [],
        image: "img/photoFrame.svg",
        backgroundPhoto: "img/postBg.png",
        phrase: ""
      }
    }


    /**
     * Creates a new group for this user
     * @param groupForm {{
     * userHref:string,
     * name:string,
     * privacy:string,
     * description:string,
     * url:string|null}}
     * @returns {Promise}
     */
    function createGroup(groupForm) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupRepository/actions/addGroup/invoke', {},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).save({
        userSubscription: {value: {href: groupForm.userHref}},
        groupName: {value: groupForm.name},
        privacy: {value: groupForm.privacy},
        description: {value: groupForm.description},
        url: {value: groupForm.url}
      }).$promise;
    }

    /**
     * Updates the group's name, description and visibility
     * @param groupForm {{
     * groupInstanceHrefId:string,
     * name:string,
     * description:string,
     * privacy:string
     * }}
     * @returns {Promise}
     */
    function updateGroupFields(groupForm) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.Group/:objectId/actions/changes/invoke',
        {objectId: groupForm.groupInstanceHrefId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1;'}
          }
        }
      ).save({
        groupName: {value: groupForm.name},
        description: {value: groupForm.description},
        privacy: {value: groupForm.privacy}
      }).$promise;
    }

    /**
     * Updates the group's photo
     * @param groupInstanceId {number|string}
     * @param photoLocation {string}
     * @returns {Promise}
     */
    function updateGroupsImage(groupInstanceId, photoLocation) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.Group/:objectId/actions/changeUrl/invoke',
        {objectId: groupInstanceId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).save({
        url: {value: photoLocation}
      }).$promise;
    }

    /**
     * Updates the group's background
     * @param groupInstanceId {string}
     * @param photoLocation {string}
     * @returns {promise}
     */
    function updateGroupsBackground(groupInstanceId, photoLocation) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.Group/:objectId/actions/changeBackgroundUrl/invoke',
        {objectId: groupInstanceId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).save({
        backgroundUrl: {value: photoLocation}
      }).$promise;
    }

    /**
     * Removes a group owned by this user
     * @param groupId {string}
     * @returns {Promise}
     */
    function removeGroup(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.Group/:objectId/actions/delete/invoke',
        {objectId: groupId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).save().$promise;
    }

    /**
     * Adds a member to a given group
     * @param data {{groupId:string, adminHref: string, friendHref:string}}
     * @returns {Promise}
     */
    function addMember(data) {
      return $resource(
        //to use with old versions
        //ApiEndPoint.url + 'restful/objects/simple.Group/:objectId/actions/addGroupMember/invoke',
        //to use with version 1711
        ApiEndPoint.url + 'restful/objects/simple.Group/:objectId/actions/addGroupMemberv1711/invoke',
        {objectId: data.groupId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).save({
        userSubscriptionAdm: {value: {href: data.adminHref}},
        userSubscription: {value: {href: data.friendHref}}
      }).$promise;
    }

    /**
     * Gets a list of the members of a group (active, banned, blocked, etc)
     * @param groupId {number|string}
     * @returns {Promise}
     */
    function getGroupMembersOld(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findMembersByGroupId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {groupId: {value: (typeof groupId === 'string' ? groupId : groupId.toString())}}
      }).$promise;
    }

    /**
     * @param groupId {number|string}
     * @return {*}
     */
    function countMembersByGroupIdWS(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/countMembersByGroupIdv1106/invoke',
        {},
        {
          get: {
            method: 'GET',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        groupId: '"' + groupId + '"'
      });
    }

    /**
     * Gets a list of the members of a group (active, banned, blocked, etc)
     * @param data {{groupId: number|string, start: number, count: number}}
     * @returns {Promise}
     */
    function getGroupMembersWS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findMembersByGroupIdv1106/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          groupId: {value: data.groupId.toString()},
          withStart: {value: data.start.toString()},
          withCount: {value: data.count.toString()}
        }
      }).$promise;
    }

    /**
     * Gets a list of the members of a group (active, banned, blocked, etc)
     * @param data {{groupId: number|string, start: number, count: number}}
     * @returns {Promise}
     */
    function getSpecificGroupMembersWS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findMembersByGroupIdv2/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          groupId: {value: data.groupId.toString()},
          withStart: {value: data.start.toString()},
          withCount: {value: data.count.toString()},
          parSearch: {value: data.parSearch}
        }
      }).$promise;
    }

    /**
     * Gets the user's requests (pending) to join to private groups
     * @param data {{ownerSubscription:string, groupSubscription:string}} - the owner and group href string subscriptions
     * @returns {*}
     */
    function createGroupInvitation(data) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserInvitationCodeRepository/actions/addUserInvitationGroup/invoke',
        {},
        {
          get: {
            method: 'GET',
            //isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubscription: {value: {href: data.ownerSubscription}},
          group: {value: {href: data.groupSubscription}}
        }
      });
    }

    /**
     * Gets the user's requests (pending) to join to private groups
     * @param userHref {string} the user href string subscription
     * @returns {Promise}
     */
    function getUserJoinRequests(userHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByUserSubscription/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {userSubscription: {value: {href: userHref}}}
      }).$promise;
    }

    /**
     * Gets the requests (pending to approve) for joining to the user's groups
     * @param data {{groupId: number, start?: number, count?: number}}
     * @returns {Promise}
     */
    function getRequestsToAproveWS(data) {
      return $resource(
        //older versions
        //ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByAdministrator/invoke',
        //ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByAdministratorv1711/invoke',
        //newest version
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByAdministratorv1712/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          //userSubscription: {value: {href: data.userHref}},//<-- for version findPendingRequestByAdministratorv1711 and earlier
          groupId: {value: data.groupId.toString()},
          withStart: {value: data.start ? data.start.toString() : '0'},
          withCount: {value: data.count ? data.count.toString() : '0'}
        }
      });
    }

    /**
     * Gets the requests (pending to approve) for joining to the user's groups
     * @param data {{groupId: number, start?: number, count?: number}}
     * @returns {Promise}
     */
    function getSpecificRequestsToApproveWS(data) {
      return $resource(
        //older versions
        //ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByAdministrator/invoke',
        //ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByAdministratorv1711/invoke',
        //newest version
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findPendingRequestByAdministratorv1713/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          //userSubscription: {value: {href: data.userHref}},//<-- for version findPendingRequestByAdministratorv1711 and earlier
          groupId: {value: data.groupId.toString()},
          withStart: {value: data.start ? data.start.toString() : '0'},
          withCount: {value: data.count ? data.count.toString() : '0'},
          search: {value: data.search ? data.search : ''}
        }
      });
    }


    /**
     * Gets a list of users whom invited to guest user
     * @param guestUser {string}
     * @param groupId {string}
     * @returns {Promise}
     */
    function findUsersWhoInvitedWS(guestUser, groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserInvitationCodeRepository/actions/findUserInvitationsByUserSubscription/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubcription: {value: {href: guestUser}},
          groupId: {value: groupId.toString()}
        }
      });
    }

    /**
     *
     *
    **/
    function getAllDevicesFromGroup(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findUserDeviceByGroupId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1;suppress=true'}
          }
        }
      ).get({
        groupId: '"' + groupId + '"'
      });
    }

    /**
     * Approves a join request for a user group
     * @param groupMermberId {number|string}
     * @returns {Promise}
     */
    function approveJoinRequest(groupMermberId) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.GroupMember/:objectId/actions/approved/invoke',
        {objectId: groupMermberId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).post().$promise;
    }

    /**
     *
     *Approves all the request waiting for a user group
     *
     *
    **/
    function approveAllJoinRequests(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/approvedAllMembersPendingRequestByGroupId/invoke',
        {},
        {
          get: {
            method: 'GET',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1;suppress=true'}
          }
        }
      ).get({
        groupId: '"' + groupId + '"'
      });
    }

    /**
     * Gets the number or joining request for a given group
     * @param groupId {number|string}
     * @returns {Resource}
     */
    function getNumJoiningRequestsByGroup(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findCountPendingRequestByGroupId/invoke',
        {},
        {
          get: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        groupId: {value: groupId.toString()}
      });
    }

    /**
     * Removes a user from a group
     * @param groupHrefId {number|string}
     * @returns {Promise}
     */
    function removeMember(groupHrefId) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.GroupMember/:objectId/actions/delete/invoke',
        {objectId: groupHrefId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).post().$promise
    }

    /**
     * Updates 'administrator' field (true|false) in GroupUser class
     * @param userHrefId {number|string}
     * @param willBeAdmin {boolean}
     * @returns {Promise}
     */
    function changeAdministrator(userHrefId, willBeAdmin) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.GroupMember/:objectId/actions/changeAdministrator/invoke',
        {objectId: userHrefId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).post({
        administrator: {value: willBeAdmin}
      }).$promise;
    }

    /**
     * Updates 'moderator' field (true|false) in GroupUser class
     * @param userHrefId {number|string}
     * @param willBeModerator {boolean}
     * @returns {Promise}
     */
    function changeModerator(userHrefId, willBeModerator) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.GroupMember/:objectId/actions/changeModerator/invoke',
        {objectId: userHrefId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).post({
        moderator: {value: willBeModerator}
      }).$promise;
    }

    /**
     *
     * @param userHref {string}
     * @param groupHref {string}
     * @returns {Promise}
     */
    function banUserFromGroup(userHref, groupHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserGroupBannedRepository/actions/addUserGroupBanned/invoke',
        {},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).post({
        userSubscription: {value: {href: userHref}},
        group: {value: {href: groupHref}}
      }).$promise;
    }

    /**
     *
     * @param groupId {number|string}
     * @returns {Promise}
     */
    function getBannedUsersByGroup(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserGroupBannedRepository/actions/findUserGroupBannedByGroupId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).get({
        "x-isis-querystring": {groupId: {value: (typeof groupId === 'string' ? groupId : groupId.toString())}}
      }).$promise;
    }

    /**
     * Gets the groups which a user has been banned from
     * @param userHref {string}
     * @returns {Promise}
     */
    function getBannedGroupsForUser(userHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserGroupBannedRepository/actions/findUserGroupBannedByUserSubscription/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {userSubscription: {value: {href: userHref}}}
      }).$promise;
    }

    /**
     *
     * @param userHref {string}
     * @param groupHref {string}
     * @returns {Promise}
     */
    function lockUserFromGroup(userHref, groupHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserGroupLockedRepository/actions/addUserGroupLocked/invoke',
        {},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).post({
        userSubscription: {value: {href: userHref}},
        group: {value: {href: groupHref}}
      }).$promise;
    }

    /**
     * Unlocks a user given its subsription to the locked bucket
     * @param userGroupLockedHrefId {string}
     * @returns {Promise}
     */
    function unlockUserFromGroup(userGroupLockedHrefId) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.UserGroupLocked/:objectId/actions/delete/invoke',
        {objectId: userGroupLockedHrefId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).post().$promise;
    }

    /**
     *
     * @param groupId {number|string}
     * @returns {Promise}
     */
    function getLockedUsersByGroup(groupId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserGroupLockedRepository/actions/findUserGroupLockedByGroupId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }).get({
        "x-isis-querystring": {groupId: {value: (typeof groupId === 'string' ? groupId : groupId.toString())}}
      }).$promise;
    }

    /**
     * Gets the groups which a user has been banned from
     * @param userHref {string}
     * @returns {Promise}
     */
    function getLockedGroupsForUser(userHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/UserGroupLockedRepository/actions/findUserGroupLockedByUserSubscription/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {userSubscription: {value: {href: userHref}}}
      }).$promise;
    }

    function gelAllGroups() {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupRepository/actions/findAllGroups/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get().$promise;
    }

    /**
     * gets a list of a user groups
     * @param userHref {number|string}
     * @returns {Promise}
     */
    function getUserGroups(userHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findGroupByUserSubscription/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubscription: {value: {href: userHref}}
        }
      }).$promise;
    }

    /**
     * gets a list of a user groups
     * @param data {{userSubscriptionHref:string, start: number, count: number}}
     * @returns {Promise}
     */
    function getUserGroupsWS(data) {
      return $resource(
        // For old versions
        // ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findGroupByUserSubscription/invoke',
        //for latest version
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findGroupByUserSubscriptionv1711/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubscription: {value: {href: data.userSubscriptionHref}},
          //for using with findGroupByUserSubscriptionv1711 version
          withStart: {value: data.start.toString()},
          withCount: {value: data.count.toString()}
        }
      }).$promise;
    }


    /**
     * Gets a list of the latest sequences of a group
     * @param groupId:number,
     * @param pagination {{start:number, count:number}}
     * @returns {Promise}
     */
    function getGroupSequences(groupId, pagination) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/findLatestSequencesBelongingToAGroupId/invoke', {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        groupId: '"' + groupId + '"',
        withStart: '"' + pagination.start + '"',
        withCount: '"' + pagination.count + '"'
      }).$promise;
    }

    /**
     * Gets a list of the latest sequences of a group
     * @param groupId:number,
     * @param pagination {{start:number, count:number}}
     * @returns {Promise}
     */
    function getSpecificGroupSequences(groupId, pagination, city, searched) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/findLatestSequencesBelongingToAGroupIdv203/invoke', {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        groupId: '"' + groupId + '"',
        withStart: '"' + pagination.start + '"',
        withCount: '"' + pagination.count + '"',
        city: '"' + city + '"',
        description: '"' + searched + '"'
      }).$promise;
    }

    function getSectionGroupSequences(groupId, pagination, city, searched, userSubs) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/findLatestSequencesBelongingToAGroupIdv204/invoke', {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        groupId: '"' + groupId + '"',
        withStart: '"' + pagination.start + '"',
        withCount: '"' + pagination.count + '"',
        city: '"' + city + '"',
        description: '"' + searched + '"',
        userSubscriptionId: '"' + userSubs + '"'
      }).$promise;
    }

    /**
     * Gets the plublic and private groups where the user is not subscribed to
     * @param data {{userSubscriptionHref:string, start:number, count:number}}
     * @returns {Promise}
     */
    function getOtherGroupsWS(data) {
      return $resource(
        //for old versions
        //ApiEndPoint.url + 'restful/services/GroupRepository/actions/findAllPublicAndPrivateGroupsByUserSubscription/invoke', {},
        //for latest version
        ApiEndPoint.url + 'restful/services/GroupRepository/actions/findAllPublicAndPrivateGroupsByUserSubscriptionv1711/invoke', {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubscription: {value: {href: data.userSubscriptionHref}},
          //for latest version
          withStart: {value: data.start.toString()},
          withCount: {value: data.count.toString()}
        },
      }).$promise;
    }

    /**
     * Shares a group timeline cqnz to the user's default timeline
     * @param cqnzId {string|number}
     * @param userHref {string|number}
     * @returns {Promise}
     */
    function shareGroupCqnzToTimeline(cqnzId, userHref) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceRepository/actions/shareGroupToTimeLine/invoke',
        {},
        {
          get: {
            method: 'GET',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          sequenceId: {value: cqnzId.toString()},
          userSubscription: {value: {href: userHref}}
        }
      }).$promise;
    }

    /**
     * Shares a cqnz to a group's timeline
     * @param cqnzId {string|number}
     * @param groupHrefId {string|number}
     * @returns {Promise}
     */
    function shareGroupCqnzToGroup(cqnzId, groupHrefId) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceRepository/actions/shareGroupToGroup/invoke',
        {},
        {
          get: {
            method: 'GET',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          sequenceId: {value: cqnzId.toString()},
          groupId: {value: groupHrefId.toString()}
        }
      }).$promise;
    }

    function findSequenceGroupsWS(sequenceId, userHref) {
      //console.log(sequenceId, userHref);
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/findSequencePermit/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          sequenceId: {value: sequenceId.toString()},
          userSubscription: {value: {href: userHref}}
        }
      });
    }

    /**
     * Verifies whether a group is in the given black list or not
     * @param blackList {Array}
     * @param groupHref {string}
     * @returns {boolean}
     */
    function checkIfIsBannedOrLockedGroup(blackList, groupHref) {
      var found = false;

      blackList.some(function (resource) {
        if (resource.group && resource.group.href === groupHref)
          return found = true;
      });

      return found;
    }

    /**
     * Verifies whether a user is in the given black list or not
     * @param blackList {Array}
     * @param userHref {string}
     * @returns {boolean}
     */
    function checkIfIsBannedOrLockedUser(blackList, userHref) {
      return blackList.some(function (user) {
        return user.userSubscription && user.userSubscription.href === userHref;
      });
    }

    function changeEnableComments(groupData) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.Group/{objectId}/actions/changeEnableComment/invoke',
        {objectId: groupData.groupInstanceId},
        {
          post: {
            method: 'POST',
            headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1;'}
          }
        }
      ).save({
        habilitar: {value: groupData.toValue}
      })
    }

    /*
    * get endpoints from your followers
    */
    function getEndPointsFollowers(userData) {
      return $resource(
        ApiEndPoint.url + 'restful/services/FriendRepository/actions/findUserDeviceUserSubscriptionId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubscriptionId: {value: userData.userSubscriptionId.toString()}
        }
      }).$promise;
    }

    /*
    * get endpoints from your followers WITH TYPE OF NOTIF
    */
    function getEndPointsFollowerswithNotif(userData) {
      return $resource(
        ApiEndPoint.url + 'restful/services/FriendRepository/actions/findFollowersUserDeviceUserSubscriptionId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          userSubscriptionId: {value: userData.userSubscriptionId.toString()},
          notificationType: {value: userData.notification.toString()}
        }
      }).$promise;
    }

    /*
    * get endpoints from the group except yourself
    */
    function getEndPointMembers(userData) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findUserDeviceByGroupIdAndUserSubscriptionId/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          groupId: {value: userData.groupId.toString()},
          userSubscriptionId: {value: userData.userSubscriptionId.toString()}
        }
      }).$promise;
    }

    /*
    * get endpoints from the group except yourself
    */
    function getEndPointMemberswithNotif(userData) {
      return $resource(
        ApiEndPoint.url + 'restful/services/GroupMemberRepository/actions/findUserDeviceByGroupIdAndUserSubscriptionIdv200/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          groupId: {value: userData.groupId.toString()},
          userSubscriptionId: {value: userData.userSubscriptionId.toString()},
          notificationType: {value: userData.notification.toString()}
        }
      }).$promise;
    }

    function notifyMembersNewPost(groupData) {
      //sentNotifications
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/sentNotifications/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).get({
        "x-isis-querystring": {
          sequence: { value: { href: groupData.sequence.toString() } },
          groupId: { value: groupData.groupId.toString() }
        }
      }).$promise;
    }

    return {
      getGroupForm: function () {
        return groupForm
      },
      resetGroupForm: resetGroupForm,
      createGroup: createGroup,
      updateGroupFields: updateGroupFields,
      updateGroupsPhoto: updateGroupsImage,
      updateGroupsBackground: updateGroupsBackground,
      removeGroup: removeGroup,
      addMember: addMember,
      getNumJoiningRequestsByGroup: getNumJoiningRequestsByGroup,
      removeMember: removeMember, //used for leaving the group too
      countMembersByGroupIdWS: countMembersByGroupIdWS,
      getGroupMembersWS: getGroupMembersWS,
      getSpecificGroupMembersWS: getSpecificGroupMembersWS,
      createGroupInvitation: createGroupInvitation,
      getUserJoinRequests: getUserJoinRequests,
      getRequestsToAproveWS: getRequestsToAproveWS,
      getSpecificRequestsToApproveWS: getSpecificRequestsToApproveWS,
      findUsersWhoInvitedWS: findUsersWhoInvitedWS,
      getAllDevicesFromGroup: getAllDevicesFromGroup,
      approveJoinRequest: approveJoinRequest,
      approveAllJoinRequests: approveAllJoinRequests,
      //findAdministrators: findAdministrators,
      changeAdministrator: changeAdministrator,
      changeModerator: changeModerator,
      banUserFromGroup: banUserFromGroup,
      getBannedUsersByGroup: getBannedUsersByGroup,
      getBannedGroupsForUser: getBannedGroupsForUser,
      getLockedGroupsForUser: getLockedGroupsForUser,
      lockUserFromGroup: lockUserFromGroup,
      unlockUserFromGroup: unlockUserFromGroup,
      getLockedUsersByGroup: getLockedUsersByGroup,
      gelAllGroups: gelAllGroups,
      getUserGroups: getUserGroups,
      getUserGroupsWS: getUserGroupsWS,
      getOtherGroupsWS: getOtherGroupsWS,
      getGroupSequences: getGroupSequences,
      getSpecificGroupSequences: getSpecificGroupSequences,
      getSectionGroupSequences: getSectionGroupSequences,
      shareGroupCqnzToTimeline: shareGroupCqnzToTimeline,
      shareGroupCqnzToGroup: shareGroupCqnzToGroup,
      findSequenceGroupsWS: findSequenceGroupsWS,
      checkIfIsBannedOrLockedGroup: checkIfIsBannedOrLockedGroup,
      checkIfIsBannedOrLockedUser: checkIfIsBannedOrLockedUser,
      getEndPointsFollowers: getEndPointsFollowers,
      getEndPointsFollowerswithNotif: getEndPointsFollowerswithNotif,
      getEndPointMembers: getEndPointMembers,
      getEndPointMemberswithNotif: getEndPointMemberswithNotif,
      notifyMembersNewPost: notifyMembersNewPost
    }
  }]);

/**
 * @typedef {Object} GroupMemberWSType
 * @property {boolean} administrator
 * @property {boolean} banned
 * @property {{href:string}} groupMember
 * @property {string} groupName
 * @property {boolean} locked
 * @property {boolean} moderator
 * @property {(string|null)} profilePictureBackgroundUrl
 * @property {(string|null)} profilePictureUrl
 * @property {string} userBio
 *  @property {{href:string}} userGroupLocked - only if user has been locked
 * @property {{href:string}} userSubscription
 */
